@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 """oschmod module.
 
-Module for working with file permissions.
+Module for working with file permissions that are consistent across Windows,
+macOS, and Linux.
 
 These bitwise permissions from the stat module can be used with this module:
     stat.S_IRWXU   # Mask for file owner permissions
@@ -27,10 +28,10 @@ These bitwise permissions from the stat module can be used with this module:
 import os
 import platform
 import random
+import re
 import stat
 import string
 
-
 IS_WINDOWS = platform.system() == 'Windows'
 HAS_PYWIN32 = False
 try:
@@ -153,7 +154,7 @@ STAT_KEYS = (
     "S_IXOTH"
 )
 
-__version__ = "0.2.7"
+__version__ = "0.3.0"
 
 
 def get_mode(path):
@@ -164,10 +165,26 @@ def get_mode(path):
 
 
 def set_mode(path, mode):
-    """Get bitwise mode (stat) of object (dir or file)."""
+    """
+    Set bitwise mode (stat) of object (dir or file).
+
+    Three types of modes can be used:
+    1. Decimal mode - an integer representation of set bits (eg, 512)
+    2. Octal mode - a string expressing an octal number (eg, "777")
+    3. Symbolic representation - a string with modifier symbols (eg, "+x")
+    """
+    new_mode = 0
+    if isinstance(mode, int):
+        new_mode = mode
+    elif isinstance(mode, str):
+        if '+' in mode or '-' in mode or '=' in mode:
+            new_mode = get_effective_mode(get_mode(path), mode)
+        else:
+            new_mode = int(mode, 8)
+
     if IS_WINDOWS:
-        return win_set_permissions(path, mode)
-    return os.chmod(path, mode)
+        return win_set_permissions(path, new_mode)
+    return os.chmod(path, new_mode)
 
 
 def set_mode_recursive(path, mode, dir_mode=None):
@@ -203,6 +220,51 @@ def set_mode_recursive(path, mode, dir_mode=None):
     return set_mode(path, dir_mode)
 
 
+def _get_effective_mode_multiple(current_mode, modes):
+    """Get octal mode, given current mode and symbolic mode modifiers."""
+    new_mode = current_mode
+    for mode in modes.split(","):
+        new_mode = get_effective_mode(new_mode, mode)
+    return new_mode
+
+
+def get_effective_mode(current_mode, symbolic):
+    """Get octal mode, given current mode and symbolic mode modifier."""
+    if not isinstance(symbolic, str):
+        raise AttributeError('symbolic must be a string')
+
+    if "," in symbolic:
+        return _get_effective_mode_multiple(current_mode, symbolic)
+
+    result = re.search(r'^\s*([ugoa]*)([-+=])([rwx]*)\s*$', symbolic)
+    if result is None:
+        raise AttributeError('bad format of symbolic representation modifier')
+
+    whom = result.group(1) or "ugo"
+    operation = result.group(2)
+    perm = result.group(3)
+
+    if "a" in whom:
+        whom = "ugo"
+
+    # bitwise magic
+    bit_perm = _get_basic_symbol_to_mode(perm)
+    mask_mode = ("u" in whom and bit_perm << 6) | \
+        ("g" in whom and bit_perm << 3) | \
+        ("o" in whom and bit_perm << 0)
+
+    if operation == "=":
+        original = ("u" not in whom and current_mode & 448) | \
+            ("g" not in whom and current_mode & 56) | \
+            ("o" not in whom and current_mode & 7)
+        return mask_mode | original
+
+    if operation == "+":
+        return current_mode | mask_mode
+
+    return current_mode & ~mask_mode
+
+
 def get_object_type(path):
     """Get whether object is file or directory."""
     object_type = DIRECTORY
@@ -302,6 +364,13 @@ def win_get_permissions(path):
     return _win_get_permissions(path, get_object_type(path))
 
 
+def _get_basic_symbol_to_mode(symbol):
+    """Calculate numeric value of set of 'rwx'."""
+    return ("r" in symbol and 1 << 2) | \
+        ("w" in symbol and 1 << 1) | \
+        ("x" in symbol and 1 << 0)
+
+
 def _win_get_permissions(path, object_type):
     """Get the permissions."""
     sec_des = win32security.GetNamedSecurityInfo(
